@@ -105,7 +105,7 @@ class dtifitOptions {
   Option<int> y_max;
   Option<int> x_min;
   Option<int> x_max;
-  FmribOption<string> grad_file;
+  Option<string> grad_file;
   FmribOption<bool> save_bvals;
   bool parse_command_line(int argc, char** argv);
   
