@@ -80,6 +80,9 @@ const float maxfloat=1e10;
 const float minfloat=1e-10;
 const float maxlogfloat=23;
 const float minlogfloat=-23;
+
+#define UPPERDIFF 0.005
+
 namespace FIBRE{
   
 
@@ -149,16 +152,17 @@ namespace FIBRE{
     ColumnVector m_Signal_old; 
     const float& m_d;
     const float& m_d_std;         //second d param in multi-shell model - std in gamma dist of diffusivities
+    const float& m_R;             //R=Perpendicular/Axial diffusivity in model3 
     const ColumnVector& m_alpha;  //Theta angles of bvecs 
     const ColumnVector& m_beta;   //Phi angles of bvecs 
     const Matrix& m_bvals;        //b values
-    const int& m_modelnum;        //1 for single-shell, 2 for multi-shell model 
+    const int& m_modelnum;        //1 for deconvolution with sticks, 2 for deconvolution with sticks and a Gamma distribution of diffusivities, 3 for deconvolution with zeppelins
  public:
     //constructors::
 
     Fibre( const ColumnVector& alpha, const ColumnVector& beta, 
-	   const Matrix& bvals,const float& d,const float& ardfudge=1,const int& modelnum=1, const float& d_std=0.01):
-      m_ardfudge(ardfudge), m_d(d),m_d_std(d_std), m_alpha(alpha), m_beta(beta), m_bvals(bvals), m_modelnum(modelnum){
+	   const Matrix& bvals,const float& d,const float& ardfudge=1,const int& modelnum=1, const float& d_std=0.01,const float& R=0.13):
+    m_ardfudge(ardfudge), m_d(d),m_d_std(d_std), m_R(R), m_alpha(alpha), m_beta(beta), m_bvals(bvals), m_modelnum(modelnum){
 	m_th=M_PI/2;
 	m_th_old=m_th;
 	m_ph=0;
@@ -202,8 +206,8 @@ namespace FIBRE{
     Fibre(const ColumnVector& alpha, 
 	  const ColumnVector& beta, const Matrix& bvals, const float& d, const float& ardfudge,
 	  const float& th, const float& ph, const float& f, 
-	  const bool lam_jump=true, const int& modelnum=1, const float& d_std=0.01) : 
-      m_th(th), m_ph(ph), m_f(f), m_ardfudge(ardfudge),m_lam_jump(lam_jump),  m_d(d), m_d_std(d_std), m_alpha(alpha), m_beta(beta), m_bvals(bvals), m_modelnum(modelnum)
+	  const bool lam_jump=true, const int& modelnum=1, const float& d_std=0.01, const float& R=0.13) : 
+    m_th(th), m_ph(ph), m_f(f), m_ardfudge(ardfudge),m_lam_jump(lam_jump),  m_d(d), m_d_std(d_std), m_R(R), m_alpha(alpha), m_beta(beta), m_bvals(bvals), m_modelnum(modelnum)
     {
       
       m_th_old=m_th;
@@ -241,7 +245,7 @@ namespace FIBRE{
      
     }
     Fibre(const Fibre& rhs): 
-      m_d(rhs.m_d), m_d_std(rhs.m_d_std), m_alpha(rhs.m_alpha), m_beta(rhs.m_beta), m_bvals(rhs.m_bvals), m_modelnum(rhs.m_modelnum){
+      m_d(rhs.m_d), m_d_std(rhs.m_d_std), m_R(rhs.m_R), m_alpha(rhs.m_alpha), m_beta(rhs.m_beta), m_bvals(rhs.m_bvals), m_modelnum(rhs.m_modelnum){
       *this=rhs;
     }
 
@@ -292,8 +296,7 @@ namespace FIBRE{
     OUT(m_lam_acc); 
     OUT(m_lam_rej);
     OUT(m_lam_jump);
-      
-
+    OUT(m_R);
     }
     
     inline const ColumnVector& getSignal() const{  
@@ -405,7 +408,7 @@ namespace FIBRE{
     void compute_signal(){
        m_Signal_old=m_Signal;
        
-       if(m_modelnum==1 || m_d_std<1e-5){
+       if(m_modelnum==1 || (m_d_std<1e-5 && m_modelnum==2)){
 	 for (int i = 1; i <= m_alpha.Nrows(); i++){
 	   float cos_alpha_minus_theta=cos(m_alpha(i)-m_th);   //Used trigonometric identities to reduce the number of sinusoids evaluated  
            float cos_alpha_plus_theta=cos(m_alpha(i)+m_th);
@@ -417,16 +420,29 @@ namespace FIBRE{
 	 }
        }
        else if(m_modelnum==2){
-	 float dbeta=m_d/(m_d_std*m_d_std);
-	 float dalpha=m_d*dbeta;                            
+	 //float dbeta=m_d/(m_d_std*m_d_std);
+	 //float dalpha=m_d*dbeta;     
+         float sig2=m_d_std*m_d_std;
+	 float dalpha=m_d*m_d/sig2;                        
 	 for (int i = 1; i <= m_alpha.Nrows(); i++){
 	   float cos_alpha_minus_theta=cos(m_alpha(i)-m_th);   
            float cos_alpha_plus_theta=cos(m_alpha(i)+m_th);
      	   float angtmp=cos(m_ph-m_beta(i))*(cos_alpha_minus_theta-cos_alpha_plus_theta)/2 + (cos_alpha_minus_theta+cos_alpha_plus_theta)/2;
  	   angtmp=angtmp*angtmp;
-           m_Signal(i)=exp(log(dbeta/(dbeta + m_bvals(1,i)*angtmp))*dalpha); //gamma distribution of diffusivities - params alpha (m_d) and beta (m_d_beta): Expected signal is (beta./(beta+b*g(th,ph))).^alpha;
+           //m_Signal(i)=exp(log(dbeta/(dbeta + m_bvals(1,i)*angtmp))*dalpha); //gamma distribution of diffusivities - params alpha (m_d) and beta (m_d_beta): Expected signal is (beta./(beta+b*g(th,ph))).^alpha;
+	   m_Signal(i)=std::exp(std::log(m_d/(m_d + m_bvals(1,i)*angtmp*sig2))*dalpha); // more stable
  	 }
        }
+       else if(m_modelnum==3){
+	 float invR=1.0/(2*m_R+1.0);
+	 for (int i = 1; i <= m_alpha.Nrows(); i++){
+	   float cos_alpha_minus_theta=cos(m_alpha(i)-m_th);   
+           float cos_alpha_plus_theta=cos(m_alpha(i)+m_th);
+     	   float angtmp=cos(m_ph-m_beta(i))*(cos_alpha_minus_theta-cos_alpha_plus_theta)/2 + (cos_alpha_minus_theta+cos_alpha_plus_theta)/2;
+ 	   angtmp=angtmp*angtmp;
+	   m_Signal(i)=exp(-m_bvals(1,i)*3*m_d*invR*((1-m_R)*angtmp+m_R));
+	 }
+       }
     }
 
 
@@ -635,6 +651,14 @@ namespace FIBRE{
     float m_d_std_acc;
     float m_d_std_rej;
     
+    float m_R;   //anisotropy parameter in model3 - R=l2/l1 for the zeppelin compartment 
+    float m_R_old;
+    float m_R_prop;
+    float m_R_prior; 
+    float m_R_old_prior;
+    float m_R_acc;
+    float m_R_rej;
+
     float m_S0;
     float m_S0_old;
     float m_S0_prop;
@@ -677,12 +701,16 @@ namespace FIBRE{
     const bool m_rician;          //If true, use Rician noise model 
     const bool m_includef0;       //If true, include an unattenuated signal compartment in the model with fraction f0
     const bool m_ardf0;           //If true, use ard on the f0 compartment 
+    const float m_R_priormean;    //Parameters to use for the Gaussian prior on R. Mean
+    const float m_R_priorstd;     //and variance
+    const float m_R_priorfudge;   //and fudge. Default is zero. If set >0, then an ARD prior is used for R at the high diffusivity regions with the requested fugde  
+
 
   public:
     //Constructor
     Multifibre(const ColumnVector& data,const ColumnVector& alpha, 
-	       const ColumnVector& beta, const Matrix& b, int N ,float ardfudge=1, int modelnum=1, bool rician=false, bool inclf0=false, bool ardf0=false):
-    m_ardfudge(ardfudge),m_data(data), m_alpha(alpha), m_beta(beta), m_bvals(b), m_modelnum(modelnum), m_rician(rician), m_includef0(inclf0), m_ardf0(ardf0){
+	       const ColumnVector& beta, const Matrix& b, int N ,float ardfudge=1, int modelnum=1, bool rician=false, bool inclf0=false, bool ardf0=false, float Rmean=0.13, float Rstd=0.03, float Rfudge=0):
+    m_ardfudge(ardfudge),m_data(data), m_alpha(alpha), m_beta(beta), m_bvals(b), m_modelnum(modelnum), m_rician(rician), m_includef0(inclf0), m_ardf0(ardf0), m_R_priormean(Rmean), m_R_priorstd(Rstd), m_R_priorfudge(Rfudge){
       m_iso_Signal.ReSize(alpha.Nrows());
       m_iso_Signal=0;
       m_iso_Signal_old=m_iso_Signal;            //Initialize vectors that keep the signal from the isotropic compartment
@@ -692,8 +720,9 @@ namespace FIBRE{
       m_S0_acc=0; m_S0_rej=0;
       m_tau_acc=0; m_tau_rej=0;
       m_f0_acc=0; m_f0_rej=0;
-      m_d_prior=0; m_d_std_prior=0; m_S0_prior=0; m_f0_prior=0; m_tau_prior=0; 
-      m_d=0; m_d_old=0; m_d_std=0; m_d_std_old=0; m_S0=0; m_S0_old=0; 
+      m_R_acc=0; m_R_rej=0;
+      m_d_prior=0; m_d_std_prior=0; m_S0_prior=0; m_f0_prior=0; m_tau_prior=0; m_R_prior=0;
+      m_d=0; m_d_old=0; m_d_std=0; m_d_std_old=0; m_S0=0; m_S0_old=0; m_R=0; m_R_old=0;
       m_f0=0.0; m_f0_old=0.0;   //Set this parameter to 0, it will be initialized later only if m_includef0 is true  
    }
     
@@ -708,19 +737,22 @@ namespace FIBRE{
     }
     
     void addfibre(const float th, const float ph, const float f,const bool use_ard=true){
-      Fibre fib(m_alpha,m_beta,m_bvals,m_d,m_ardfudge,th,ph,f,use_ard,m_modelnum,m_d_std);
+      Fibre fib(m_alpha,m_beta,m_bvals,m_d,m_ardfudge,th,ph,f,use_ard,m_modelnum,m_d_std,m_R);
        m_fibres.push_back(fib);
     }
 
     void addfibre(){
-      Fibre fib(m_alpha,m_beta,m_bvals,m_d,m_ardfudge,m_modelnum,m_d_std);
+      Fibre fib(m_alpha,m_beta,m_bvals,m_d,m_ardfudge,m_modelnum,m_d_std,m_R);
       m_fibres.push_back(fib);
     }
 
     void initialise_energies(){
       compute_d_prior();
-      if(m_modelnum==2)
+      if(m_modelnum>=2){
 	compute_d_std_prior();
+	if (m_modelnum==3)
+	  compute_R_prior();
+      }
       if (m_rician){
       	compute_tau_prior();
       }
@@ -743,6 +775,7 @@ namespace FIBRE{
       m_d_std_prop=m_d_std/10.0;
       m_tau_prop=m_tau/2.0;
       m_f0_prop=0.2;
+      m_R_prop=m_R/10.0;
     }
     
 
@@ -752,6 +785,9 @@ namespace FIBRE{
     inline float get_d_std() const{ return m_d_std;}
     inline void set_d_std(const float d_std){ m_d_std=d_std; }
 
+    inline float get_R() const{ return m_R;}
+    inline void set_R(const float R){ m_R=R; }
+
     inline int get_nfibre(){return m_fibres.size();}
     
     inline float get_energy() const { return m_likelihood_en+m_prior_en;}
@@ -782,6 +818,13 @@ namespace FIBRE{
       OUT(m_d_std_old_prior);
       OUT(m_d_std_acc);
       OUT(m_d_std_rej);
+      OUT(m_R);
+      OUT(m_R_old);
+      OUT(m_R_prop);
+      OUT(m_R_prior); 
+      OUT(m_R_old_prior);
+      OUT(m_R_acc);
+      OUT(m_R_rej);
       OUT(m_S0);
       OUT(m_S0_old);
       OUT(m_S0_prop);
@@ -801,31 +844,26 @@ namespace FIBRE{
 
     inline bool compute_d_prior(){
       m_d_old_prior=m_d_prior;
-      if(m_d<0 || m_d>0.008)
+      if(m_d<0 || m_d>UPPERDIFF)
 	return true;
       else{
-	m_d_prior=0;
+	if (m_modelnum==3){
+	  float alpha=3.0; float beta=4000;  //Gamma_prior around 0.5-1E-3
+	  m_d_prior=(1.0-alpha)*log(m_d)+beta*m_d;
+	}
+	else
+	  m_d_prior=0;
 	return false;
       }
     }
 
 
-    //More restrictive prior for the model with f0
-    //particularly useful for the CSF voxels 
-    inline bool compute_d_prior_f0(){
-      m_d_old_prior=m_d_prior;
-      if(m_d<0 || m_d>0.008)
-	return true;
-      else{
-	m_d_prior=0;
-	return false;
-      }
-    }
-
 
     inline bool compute_d_std_prior(){
+      float upper_d_std=0.01;
+      if (m_modelnum==3) upper_d_std=0.004;
       m_d_std_old_prior=m_d_std_prior;
-      if(m_d_std<=0 || m_d_std>0.01)
+      if(m_d_std<=0 || m_d_std>upper_d_std)
 	return true;
       else{
 	//m_d_std_prior=0;
@@ -833,6 +871,38 @@ namespace FIBRE{
 	return false;
       }
     }
+
+    
+    inline bool compute_R_prior(){
+      m_R_old_prior=m_R_prior;
+      
+      float upper_R=2*m_R_priormean;
+      float lower_R=m_R_priormean-2.0*m_R_priorstd; 
+
+      if (m_R_priormean>0.5)
+	upper_R=1;
+
+      if (lower_R<0)
+	lower_R=1E-8;
+      
+      if (m_R_priorfudge>0 && m_d>UPPERDIFF/2.0){ //then use an ARD prior to avoid competition with the isotropic compartments
+	if (m_R<1E-8 || m_R>upper_R)
+	  return true;
+	else{
+	  m_R_prior=m_R_priorfudge*std::log(m_R);
+	  return false;
+	}
+      }
+      if(m_R<=lower_R || m_R>upper_R)  //Truncate prior to avoid too spherical (high m_R) or too anisotropic (small m_R) profiles 
+	return true;
+      else{
+	float Rstd2=m_R_priorstd*m_R_priorstd; 
+	m_R_prior=(m_R-m_R_priormean)*(m_R-m_R_priormean)/Rstd2;  //Gaussian prior
+	return false;
+      }
+    }
+ 
+
     
     inline bool compute_S0_prior(){
       m_S0_old_prior=m_S0_prior;
@@ -884,8 +954,11 @@ namespace FIBRE{
     inline void compute_prior(){
       m_old_prior_en=m_prior_en;
       m_prior_en=m_d_prior+m_S0_prior;
-      if(m_modelnum==2)
+      if(m_modelnum>=2){
 	m_prior_en=m_prior_en+m_d_std_prior;
+	if (m_modelnum==3)
+	  m_prior_en=m_prior_en+m_R_prior;
+      }
       if(m_rician)
 	m_prior_en=m_prior_en+m_tau_prior;
       if (m_includef0)
@@ -902,11 +975,14 @@ namespace FIBRE{
 	for(int i=1;i<=m_alpha.Nrows();i++)
 	   m_iso_Signal(i)=exp(-m_d*m_bvals(1,i));
       }
-      else if(m_modelnum==2){
+      else if(m_modelnum>=2){
+	float sig2=m_d_std*m_d_std;
+	float dalpha=m_d*m_d/sig2;	  
 	for(int i=1;i<=m_alpha.Nrows();i++){
-	  float dbeta=m_d/(m_d_std*m_d_std);
-	  float dalpha=m_d*dbeta;	  
-	  m_iso_Signal(i)=exp(log(dbeta/(dbeta+m_bvals(1,i)))*dalpha);
+	  m_iso_Signal(i)=exp(log(m_d/(m_d+m_bvals(1,i)*sig2))*dalpha); // more numerically stable
+	  //float dbeta=m_d/(m_d_std*m_d_std);
+	  //float dalpha=m_d*dbeta;	  
+	  //m_iso_Signal(i)=exp(log(dbeta/(dbeta+m_bvals(1,i)))*dalpha);
 	}
       }
     }	
@@ -986,10 +1062,7 @@ namespace FIBRE{
       bool rejflag;
       m_d_old=m_d;
       m_d+=normrnd().AsScalar()*m_d_prop;
-      if (m_includef0)
-      	rejflag=compute_d_prior_f0();
-      else
-	rejflag=compute_d_prior();
+      rejflag=compute_d_prior();
       for(unsigned int f=0;f<m_fibres.size();f++)
 	m_fibres[f].compute_signal();
       compute_iso_signal();        
@@ -1017,8 +1090,10 @@ namespace FIBRE{
       m_d_std_old=m_d_std;
       m_d_std+=normrnd().AsScalar()*m_d_std_prop;
       bool rejflag=compute_d_std_prior();//inside this it stores the old prior
-      for(unsigned int f=0;f<m_fibres.size();f++)
-	m_fibres[f].compute_signal();
+      if (m_modelnum==2){
+	for(unsigned int f=0;f<m_fibres.size();f++)
+	  m_fibres[f].compute_signal();
+      }
       compute_iso_signal();   
       return rejflag;
     };
@@ -1033,13 +1108,41 @@ namespace FIBRE{
       m_d_std=m_d_std_old;
       m_d_std_prior=m_d_std_old_prior;
       m_prior_en=m_old_prior_en;
-      for(unsigned int f=0;f<m_fibres.size();f++)
-	m_fibres[f].restoreSignal();
+      if (m_modelnum==2){
+	for(unsigned int f=0;f<m_fibres.size();f++)
+	  m_fibres[f].restoreSignal();
+      }
       m_iso_Signal=m_iso_Signal_old;   
       m_d_std_rej++;
     }
     
 
+
+    inline bool propose_R(){
+      m_R_old=m_R;
+      m_R+=normrnd().AsScalar()*m_R_prop;
+      bool rejflag=compute_R_prior();//inside this it stores the old prior
+      for(unsigned int f=0;f<m_fibres.size();f++)
+	m_fibres[f].compute_signal();
+      return rejflag;
+    };
+    
+
+    inline void accept_R(){
+      m_R_acc++;      
+    }
+    
+
+    inline void reject_R(){      
+      m_R=m_R_old;
+      m_R_prior=m_R_old_prior;
+      m_prior_en=m_old_prior_en;
+      for(unsigned int f=0;f<m_fibres.size();f++)
+	m_fibres[f].restoreSignal();
+      m_R_rej++;
+    }
+
+
     inline bool propose_S0(){
       m_S0_old=m_S0;
       m_S0+=normrnd().AsScalar()*m_S0_prop;
@@ -1107,11 +1210,17 @@ namespace FIBRE{
     inline void update_proposals(){
       m_d_prop*=sqrt(float(m_d_acc+1)/float(m_d_rej+1));
       m_d_prop=min(m_d_prop,maxfloat);
-      if(m_modelnum==2){
+      if(m_modelnum>=2){
 	m_d_std_prop*=sqrt(float(m_d_std_acc+1)/float(m_d_std_rej+1));
 	m_d_std_prop=min(m_d_std_prop,maxfloat);
 	m_d_std_acc=0; 
 	m_d_std_rej=0;
+	if (m_modelnum==3){
+	  m_R_prop*=sqrt(float(m_R_acc+1)/float(m_R_rej+1));
+	  m_R_prop=min(m_R_prop,maxfloat);
+	  m_R_acc=0; 
+	  m_R_rej=0;
+	}
       }
       if (m_rician){
       	m_tau_prop*=sqrt(float(m_tau_acc+1)/float(m_tau_rej+1));
@@ -1193,7 +1302,7 @@ namespace FIBRE{
 	reject_d();
       
       
-      if(m_modelnum==2){     //Try d_std
+      if(m_modelnum>=2){     //Try d_std
 	if(!propose_d_std()){
 	  compute_prior();
 	  compute_likelihood();
@@ -1208,7 +1317,24 @@ namespace FIBRE{
 	}
 	else 
 	  reject_d_std();
-      }
+
+	if (m_modelnum==3){ //Try R
+	  if(!propose_R()){
+	    compute_prior();
+	    compute_likelihood();
+	    compute_energy();
+	    if(test_energy()){
+	      accept_R();
+	    }
+	    else{
+	      reject_R();
+	      restore_energy();
+	    }
+	  }
+	else 
+	  reject_R();
+	  }
+	} 
 
 
       if(!propose_S0()){    //Try S0
@@ -1225,8 +1351,7 @@ namespace FIBRE{
       else
 	reject_S0();
       
-      
-
+ 
 
 
       for(unsigned int f=0;f<m_fibres.size();f++){  //For each fibre
@@ -1314,6 +1439,13 @@ namespace FIBRE{
       m_d_std_old_prior=rhs.m_d_std_old_prior;
       m_d_std_acc=rhs.m_d_std_acc;
       m_d_std_rej=rhs.m_d_std_rej;
+      m_R=rhs.m_R;
+      m_R_old=rhs.m_R_old;
+      m_R_prop=rhs.m_R_prop;
+      m_R_prior=rhs.m_R_prior; 
+      m_R_old_prior=rhs.m_R_old_prior;
+      m_R_acc=rhs.m_R_acc;
+      m_R_rej=rhs.m_R_rej;
       m_S0=rhs.m_S0;
       m_S0_old=rhs.m_S0_old;
       m_S0_prop=rhs.m_S0_prop;
