@@ -62,8 +62,8 @@
     University, to negotiate a licence. Contact details are:
     innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
-#include "ptx_seedmask.h"
 #include "streamlines.h"
+#include "ptx_seedmask.h"
 #include <time.h>
 
 using namespace std;
@@ -112,6 +112,9 @@ void seedmask()
   // seed from volume-like ROIs
   if(seeds.nVols()>0){
     cout << "Volume seeds" << endl;
+    vector<int> triangles; //to avoid connections between vertices of the same triangle...but not used for volumes
+    triangles.push_back(-1);
+
     for(int roi=1;roi<=seeds.nVols();roi++){
       cout<<"volume "<<roi<<endl;
 
@@ -121,7 +124,7 @@ void seedmask()
 	for(int y=0;y<seeds.ysize();y++){
 	  for(int x=0;x<seeds.xsize();x++){
 	    if(seeds.isInRoi(x,y,z,roi)){
-	      counter.updateSeedLocation(seeds.get_volloc(roi-1,x,y,z));
+	      counter.updateSeedLocation(seeds.get_volloc(roi-1,x,y,z),-1,triangles);
 	      if(opts.verbose.value()>=1){
 		cout <<"run"<<endl;
 		cout <<x<<" "<<y<<" "<<z<<endl;
@@ -149,15 +152,20 @@ void seedmask()
 	cout << "  Using a subset of the vertices labelled active (i.e. non zero value)" << endl;
 	cout << "   set all values to 0 or non-zero to use entire surface" << endl;
       }
-
       for(int p=0;p<seeds.get_mesh(i).nvertices();p++){
 	// check if active point	
 	if(seeds.get_mesh(i).get_pvalue(p)==0.0)
 	  continue;
-	
-	counter.updateSeedLocation(seeds.get_surfloc(i,p));
-	pos=seeds.get_vertex_as_vox(i,p);
 
+        //to avoid connections between vertices of the same triangle
+	CsvMpoint vertex=seeds.get_mesh(i).get_point(p);
+	vector<int> triangles;
+	for(int t=0;t<vertex.ntriangles();t++){
+	  triangles.push_back(vertex.get_trID(t));
+	}
+	
+	counter.updateSeedLocation(seeds.get_surfloc(i,p),i,triangles);
+	pos=seeds.get_vertex_as_vox(i,p);
 
 	ColumnVector dir(3);
 	dir=seeds.get_normal_as_vox(i,p);
@@ -169,7 +177,7 @@ void seedmask()
 	  cout <<"run"<<endl;
 	  cout <<pos(1)<<" "<<pos(2)<<" "<<pos(3)<<endl;
 	}
-	  keeptotal += seedmanager.run(pos(1),pos(2),pos(3),
+	keeptotal += seedmanager.run(pos(1),pos(2),pos(3),
 				       false,-1,opts.sampvox.value());
 	
 
